#include "amba_arbiter_fp.h"

using namespace master;

namespace bus{
	
	/**
	 * Represents the implementation of an Fix Priority Algorithm
	 */
		 
	void Amba_Arbiter_Fix_Priority::mount_request(AmbaRequest & req){
		req.HTRANS = IDLE;
		req.HADDR = -1;
	}
	
	Amba_Arbiter_Fix_Priority::Amba_Arbiter_Fix_Priority(sc_module_name nm, sc_clock* clk) : 
	sc_module(nm),
	clock(clk)
		{
			master_default = new AmbaMaster("master_default");
			split_master = new AmbaMaster*[17];
			highest_priority = 127;
			SC_THREAD(arbiter_thread);
			sensitive << clock->posedge_event();
			dont_initialize();
			busy_bus = false;
			exist_master_allocated = false;
		}
	
	/**
	 *  Choose the next Master who grants the bus
	 */
	bool Amba_Arbiter_Fix_Priority::alloc_master(AmbaMaster& master, Amba_Request_Type req) {
		if (!master_is_split(master)){
			if (master.getPriority() < highest_priority){
				highest_priority = 	master.getPriority();
				temp_master = &master;
				wait(SC_ZERO_TIME);
//				if (temp_master == &master){
					if(busy_bus || exist_master_allocated){
						while (true){
							wait(release_bus);
							if((master.getPriority() < actual_master->getPriority()) || !exist_master_allocated)
							{
								if (master.getPriority() < highest_priority){
									highest_priority = 	master.getPriority();
									temp_master = &master;
									wait(SC_ZERO_TIME);						
								}	
								break;
							}
						}
					}
					if (temp_master == &master){
						exist_master_allocated = true;
						busy_bus = true;
						actual_master = &master;
						actual_request_type = req;
						highest_priority = 127; 
						return true;
					} else {
						return false;
					}
//				} else return false;
			} else return false;
		}
		return false;
	} 
			
	bool Amba_Arbiter_Fix_Priority::isMaster(AmbaMaster& master){
		if (&master == actual_master)
			return true;
		else
			return false;
	} 
			
	int Amba_Arbiter_Fix_Priority::ownerBus(){
		return actual_master->getId();
	}
			
	bool Amba_Arbiter_Fix_Priority::locked_transfer(){
		return (actual_request_type == LOCKED);
	}
			
	void Amba_Arbiter_Fix_Priority::notify_release_bus(){
		busy_bus = false;
		release_bus.notify();
	}
			
	bool Amba_Arbiter_Fix_Priority::is_busy_bus(){
		return busy_bus;
	}
			
			
	void Amba_Arbiter_Fix_Priority::setBus(AmbaBus* amba_bus){
		this->amba_bus = amba_bus;
	}
			
	void Amba_Arbiter_Fix_Priority::arbiter_thread(){
	
		while (true){ 
			if(!busy_bus) 
			{
				highest_priority = 127;
	
				bool resp = 0;
				bool master = isMaster(*master_default);

				if (!master)
					resp = amba_bus->master_request_bus_non_blocking((*master_default), UNLOCKED);

				if(resp || master){
					AmbaRequest req;
					mount_request(req);
					amba_bus->master_send_controls_signals((*master_default), req);
				}
			}
				wait();
				wait(SC_ZERO_TIME);
		}
	}
			
	void Amba_Arbiter_Fix_Priority::turn_busy_bus(){
		busy_bus = true;
	}

	void Amba_Arbiter_Fix_Priority::set_locked_transfer(Amba_Request_Type reqType){
		actual_request_type = reqType;
		if (reqType == UNLOCKED){
			exist_master_allocated = false;
		}
	}
	
	void Amba_Arbiter_Fix_Priority::init_split_transaction(AmbaMaster& master, AmbaSlave& slave){
		split_master[slave.getId()] = &master;
		exist_master_allocated = false;
	}
	
	void Amba_Arbiter_Fix_Priority::end_split_transaction(AmbaSlave& slave){
		split_master[slave.getId()] = 0;
	}
	
	bool Amba_Arbiter_Fix_Priority::master_is_split(AmbaMaster& master){
		for(int i = 0; i < 17; i ++)
			if(split_master[i] == &master)
				return true;
		return false;
	}
	
	
	AmbaMaster* Amba_Arbiter_Fix_Priority::get_actual_master(){
		return actual_master;
	}

	bool Amba_Arbiter_Fix_Priority::slave_in_split_transaction(AmbaSlave& slave){
		if(split_master[slave.getId()] != 0)
			return true;
		return false;
	}
}
