#ifndef AMBAARBITERIF_H_
#define AMBAARBITERIF_H_

#include "amba_bus.h"
#include "amba_master.h"
#include "enum_master_sig.h"


using namespace master;

    class AmbaBus;
    
    
/** 
 * @brief Amba Arbiter Interface
 * @author Andre Silva
 * @author Guilherme Alvaro
 */
    
	class Amba_Arbiter_if: public sc_interface {
		 
		public:
			
			/**
			 * @brief Requests the bus for the AmbaMaster passed
			 * @param AmbaMaster& master the master device
			 * @param Amba_Request_Type req Type of Request
			 * @return bool if the AmbaMaster get the bus
			 */
			virtual bool alloc_master(AmbaMaster& master, Amba_Request_Type req) = 0;

			/**
			 * @brief Verifies if the AmbaMaster is actual master of the bus
			 * @param AmbaMaster& master the master device
			 * @return bool if the AmbaMaster is actual master of the bus
			 */
			virtual bool isMaster(AmbaMaster& master) = 0;
			
			/**
			 * @brief Returns  the actual master of the bus
			 * @return int The ID of the actual master of the bus
			 */			
			virtual int ownerBus() = 0; 

			/**
			 * @brief Returns the type of the actual transfer
			 * @returns bool the type of the actual transfer
			 */			
			virtual bool locked_transfer() = 0;
			
			/**
			 * @brief sets the locked_transfer internal variable
			 * @param req New value for the locked_transfer variable
			 */
			virtual void set_locked_transfer(Amba_Request_Type req) = 0;
			
			/**
			 * @brief Notifies the arbiter about the end of transfer
			 */			
			virtual void notify_release_bus() = 0;

			/**
			 * @brief Sets the bus managed by arbiter
			 * @param AmbaBus* amba_bus Pointer for the new AmbaBus
			 */			
			virtual void setBus(AmbaBus* amba_bus) = 0;

			/**
			 * @brief Verifies if the bus is busy
			 * @return bool The actual state of the bus
			 */			
			virtual bool is_busy_bus() = 0;
		
			/**
			 * @brief Changes the actual state of bus for busy 
			 */			
			virtual void turn_busy_bus() = 0;

			/**
			 * @brief Inits one split transaction
			 * @param AmbaMaster& master The actual master of the bus
			 * @param AmbaSlave& slave The Slave that requests the Split transaction
			 */			
			virtual void init_split_transaction(AmbaMaster& master, AmbaSlave& slave) = 0;

			/**
			 * @brief Ends one split transaction
			 * @param AmbaSlave& slave The Slave that initiated the Split transaction
			 */			
			virtual void end_split_transaction(AmbaSlave& slave) = 0;

			/**
			 * @brief Returns the actual Master of the bus
			 * @return AmbaMaster* Pointer for the actual master of the bus
			 */			
			virtual AmbaMaster* get_actual_master() = 0;
			
			/**
			 * @brief Returns if the slave passed is in a split transaction
			 * @return bool if the slave passed is in a split transaction
			 */			
			virtual bool slave_in_split_transaction(AmbaSlave& slave) = 0;
	};

#endif /*AMBAARBITERIF_H_*/
