#include "amba_decoder.h"

AmbaDecoder::AmbaDecoder(sc_module_name nm) : sc_module(nm) {
	initSlaveArray();
	slave_default = new AmbaSlave("slave_default", 1024);
	createAmbaSlave_default(slave_default);
	slaveArray[16] = 1;	

	this->arraySlave = new AmbaSlave*[n_max_master];
	this->HSEL = n_max_master - 1;
	this->arraySlave[n_max_master-1] = slave_default;

	
	SC_THREAD(slave_default_thread);
	sensitive << alert_slave_default; 
	dont_initialize(); 
}

void AmbaDecoder::registerSlave(AmbaSlave* slave){ 
	if (check_connection(slave)){
		for (int i = 0; i < n_max_master-1; ++i) {
			if(!slaveArray[i]) 
			{ 
				this->arraySlave[i] = slave; 
				this->slaveArray[i] = 1;
				slave->setId(i);
				break;
			}
			if(i==15){
				exit(-1);
			}
		}
	}
}
		

void AmbaDecoder::deregisterSlave(AmbaSlave* slave){
	for (int i = 0; i < n_max_master-1; i++)
		if(slaveArray[i] && slave->getId() == this->arraySlave[i]->getId()){
			this->arraySlave[i] = 0;
			this->slaveArray[i] = 0;
		}
}

int AmbaDecoder::getPos(int address){
	
	int tempBase1 = 0;
	int tempOffset1 = 0;
	for (int i = 0; i < 16; ++i) {
		if(slaveArray[i]){
			tempBase1 = arraySlave[i]->getBase();
			tempOffset1 = tempBase1 + arraySlave[i]->getOffset() - 1;
			if((address >= tempBase1) && (address <= tempOffset1)){
				return i;
			}
		}
	}
	return n_max_master-1;
}


void AmbaDecoder::decoder_address(AmbaRequest& req){
	this->HSEL = getPos(req.HADDR);
}


int AmbaDecoder::get_actual_slave(){
	return this->arraySlave[this->HSEL]->getId(); 
}

void AmbaDecoder::slave_default_thread(){
	while(true){

		AmbaRequest request;
		AmbaResponse resp;
		prepare_response(resp);
		this->amba_bus->slave_receive_controls_signals(*(this->slave_default), request);
		if (request.HTRANS != IDLE){
			AmbaData data(request.HSIZE);
			
			if(request.HWRITE = true)
				this->amba_bus->slave_read_data(*(this->slave_default), resp, data);
			else
				this->amba_bus->slave_send_data(*(this->slave_default), resp, data);
		}
	}
}

void AmbaDecoder::prepare_response(AmbaResponse& resp){
	resp.HREADY = true;
	resp.HRESP = ERR;
}

void AmbaDecoder::setBus(AmbaBus* amba_bus){
	this->amba_bus = amba_bus;
}


int AmbaDecoder::n_max_master = 17;


void AmbaDecoder::initSlaveArray(){

	for(int i=0; i < 17; i++)
	{
		slaveArray[i] = 0;
	}
}


void AmbaDecoder::createAmbaSlave_default(AmbaSlave* slave_default){
		slave_default->setId(0);
		slave_default->setBase(16 * 1024);
		//cout << slave_default->getId() << " id do slave " << endl;
}
AmbaDecoder::~AmbaDecoder()
{
	
}

bool AmbaDecoder::check_connection(AmbaSlave* slave){
	int tempBase1 = slave->getBase();
	int tempOffset1 = slave->getOffset();
		
	int tempBase2 = 0;
	int tempOffset2 = 0;
	
	for (int i = 0; i < 16; ++i) {
		if(slaveArray[i]){
			tempBase2 = this->arraySlave[i]->getBase();
			tempOffset2 = this->arraySlave[i]->getOffset();

			if(
					((tempBase1 < tempBase2) && (tempOffset1) < tempBase2)
			        || 
			        ((tempBase1 > tempOffset2) && (tempOffset1) > tempOffset2)
			       ){
//			       		cout << "Faixa de endereços correta." << endl;
			       }else{
			       		cout << "Verifique a faixa de endereços dos escravos." << endl;
			       		exit(1);
			       }	    
		}
	}

	return true;
}