#ifndef AMBADECODER_H_
#define AMBADECODER_H_

#include "amba_bus.h"
#include "amba_response.h"
#include "systemc.h"
#include "util.h"
#include "amba_request.h"
#include "amba_slave.h"
#include "assert.h"



using namespace util;
using namespace master;
using namespace slave;

class AmbaBus;

/**
 * @brief Class that represents the Amba Decoder
 * @author Andre Silva
 * @author Guilherme Alvaro
 */
 
class AmbaDecoder : public sc_module { 

	public:

 		SC_HAS_PROCESS(AmbaDecoder);
		/**
		 * @brief Constructor of the class AmbaDecoder
		 * @param sc_module_name nm: Name of Decoder
		 */	
		AmbaDecoder(sc_module_name nm);
		
		/**
		 * @brief Registers a slave device in the decoder
		 * @param AmbaSlave* slave: the slave device
		 */
		void registerSlave(AmbaSlave* slave);
		

		/**
		 * @brief Remove a slave device of the decoder
		 * @param AmbaSlave* slave: the slave device
		 */
		void deregisterSlave(AmbaSlave* slave);
		

		/**
		 * @brief Decode the address passed by AmbaRequest package
		 * @param AmbaRequest& req: AmbaRequest package
		 */
		void decoder_address(AmbaRequest& req);
		

		/**
		 * @brief Sets the bus managed by arbiter
		 * @param AmbaBus* amba_bus Pointer for the new AmbaBus
		 */	
		void setBus(AmbaBus* amba_bus);
		
		/**
		 * @brief Returns the ID of the slave used currently
		 * @return char ID of the Slave used currently
		 */	
		int get_actual_slave();
		

		/**
		 * @brief Returns the adress of the slave default
		 * @return Address of the slave default
		 */	
		static int get_slave_defaul_address();
		
		virtual ~AmbaDecoder();
		
		
	private:

		/**
		 * @brief Returns if the pos (id) passed is valid
		 * @param int pos: position(id)
		 * @return bool Returns if id passed is valid
		 */	

//		bool is_valid_slave(int pos);

		/**
		 * @brief Returns if the address passed is valid
		 * @param int pos: position (id)
		 * @param AmbaRequest& req: AmbaRequest Package
		 * @return Returns if the adress passed is valid
		 */	

//		bool is_valid_address(int pos, AmbaRequest& req);


		/**
		 * @brief Pointer to array of AmbaSlave
		 */	

		AmbaSlave** arraySlave;

		/**
		 * @brief ID of the actual slave used
		 */	
		int HSEL;
		

		/**
		 * @brief Array of integer that contains the position used
		 */	
		int slaveArray[17];		

		/**
		 * @brief Initializes the slaveArray
		 */	

		void initSlaveArray();
		

		/**
		 * @brief Contains the number max of master;
		 */	

		static int n_max_master;

		/**
		 * @brief Pointer to slave default
		 */	

		AmbaSlave* slave_default;

		/**
		 * @brief Pointer to the bus
		 */	
		AmbaBus* amba_bus;
		

		/**
		 * @brief Event that initializes the slave default
		 */	
		sc_event alert_slave_default;
		

		/**
		 * @brief Thread that simulates the behaviour of slave_default
		 */	
		void slave_default_thread();

		/**
		 * @brief Creates a AmbaResponse Package
		 * @param AmbaResponse& resp: The AmbaResponse package
		 */	
		void prepare_response(AmbaResponse& resp);

		/**
		 * @brief Creates a AmbaSlave default
		 * @param AmbaSlave& slave_default: The pointer to slave default
		 */	
		void AmbaDecoder::createAmbaSlave_default(AmbaSlave* slave_default);

		/**
		 * @brief Check if the address range and the ID of slave is valid 
		 * @param AmbaSlave& slave: The pointer to slave
		 */			
		bool AmbaDecoder::check_connection(AmbaSlave* slave);
		
		int AmbaDecoder::getPos(int address);
};

#endif /*AMBADECODER_H_*/
