#ifndef AMBA_MASTER_H_
#define AMBA_MASTER_H_

#include "systemc.h"

/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief master Namespace
 */
namespace master {


/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief AmbaMaster Class
 */
	class AmbaMaster: public sc_module {
	
		public:
		
       /**
		 * @brief AmbaMaster Constructor
 		 * @param[in] sc_module_name nm: name of the module
 		 * @param[in] char priority: priority of the module
		 */      
		AmbaMaster(sc_module_name nm, char priority);
			
       /**
		 * @brief AmbaMaster Constructor
 		 * @param[in] sc_module_name nm: name of the module
		 */      
		AmbaMaster(sc_module_name nm);

       /**
		 * @brief AmbaMaster Constructor
 		 * @param[in] const AmbaMaster&: reference for AmbaMaster
		 */      
		AmbaMaster(const AmbaMaster&);

       /**
		 * @brief Default AmbaMaster Constructor
		*/			
		AmbaMaster();

		~AmbaMaster();
			
       /**
		 * @brief Get ID of the AmbaMaster in Amba Bus
 		 * @return int id returned
		 */      
		int getId()const;
			
       /**
		 * @brief Get Priority of the AmbaMaster in Amba Bus
 		 * @return int priority returned
		 */      
		int getPriority()const;

       /**
		 * @brief Operator of Copy
 		 * @return int priority returned
		 */      
		AmbaMaster& operator=(const AmbaMaster &right);

       /**
		 * @brief Sets the id of AmbaMaster
 		 * @param[in] char id: new id of AmbaMaster
		 */      
		void setId(char id);
			
	protected:

       /**
		 * @brief Id of AmbaMaster in Amba Bus
		 */    
		 char id;

       /**
		 * @brief Priority of AmbaMaster in Amba Bus
		 */    
		char priority;

       /**
		 * @brief AmbaMaster Default Priority's
		 */    
		static char const default_priority;

	} ;
}

#endif /*AMBA_MASTER_H_*/
