#ifndef AMBA_TL1_MASTER_IF_H_
#define AMBA_TL1_MASTER_IF_H_

#include "amba_master.h"
#include "amba_request.h"
#include "amba_data.h"
#include "amba_response.h"

using namespace amba;
using namespace slave;

namespace master{

/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief Amba_Master_tl1_if Class
 */ 
	class Amba_Master_tl1_if : virtual public sc_interface { 

		public:

	       /**
			 * @brief Master Resquest Operation's Non-Blocking to Amba Bus
	 		 * @param[in] AmbaMaster& master: reference to a AmbaMaster Object
	 		 * @param[in] Amba_Request_Type req: request type
	 		 * @return bool Request's Status
			 */   
			virtual bool master_request_bus_non_blocking(AmbaMaster& master, Amba_Request_Type req) = 0;			
		
	       /**
			 * @brief Master Resquest Operation's Blocking to Amba Bus
	 		 * @param[in] AmbaMaster& master: reference to a AmbaMaster Object
	 		 * @param[in] Amba_Request_Type req: request type
			 */   
			virtual void master_request_bus_blocking(AmbaMaster& master, Amba_Request_Type req) = 0;
			
	       /**
			 * @brief Sends control signals to Amba Bus
	 		 * @param[in] AmbaMaster& master: reference to a AmbaMaster Object
	 		 * @param[in] Amba_Request_Type req: request type
	 		 * @return bool Status of the control's signals sended
			 */   
			virtual bool master_send_controls_signals(AmbaMaster& master, AmbaRequest& req) = 0;
			
	       /**
			 * @brief Sends control signals to Amba Bus
	 		 * @param[in] AmbaMaster& master: reference to a AmbaMaster Object
	 		 * @param[in] AmbaDara& data: referece to data that will be transmitted
	 		 * @param[in] AmbaResponse& ambaResponse: response of the master_send_data operation
			 */   
			virtual	void master_send_data(AmbaMaster& master, AmbaData& data, AmbaResponse& ambaResponse) = 0;			 
	
	       /**
			 * @brief Reads control signals from Amba Bus
	 		 * @param[in] AmbaMaster& master: reference to a AmbaMaster Object
	 		 * @param[in] AmbaDara& data: referece to data that will be received
	 		 * @param[in] AmbaResponse& ambaResponse: response of the master_read_data operation
			 */   
	    	virtual void master_read_data(AmbaMaster& master, AmbaData& data, AmbaResponse& amba_response) = 0;
	
	       /**
			 * @brief This operation releases the Amba Bus for another Master Amba
	 		 * @param[in] AmbaMaster& master: reference to a AmbaMaster Object
			 */   
			virtual void master_release_bus(AmbaMaster& master) = 0;
			
	       /**
			 * @brief Identifies if the master in the arguments is the Amba Bus Owner's
	 		 * @param[in] AmbaMaster& master: reference to a AmbaMaster Object
			 */   
			virtual bool is_master(AmbaMaster& master) = 0;
			
	       /**
			 * @brief Registers the Amba Master in Amba Bus
	 		 * @param[in] AmbaMaster& master: pointer to a AmbaMaster Object
			 */   
			virtual void register_master(AmbaMaster* master) = 0;
	};
}

#endif /*AMBA_TL1_MASTER_IF_H_*/
