#ifndef AMBAREQUEST_H_
#define AMBAREQUEST_H_

#include "enum_master_sig.h"

namespace master{

/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief AmbaRequest Class
 */
	class AmbaRequest{
		public:
	       /**
			 * @brief This is the address for where the data will be sent
			 */   
			int HADDR;

	       /**
			 * @brief 
			 */   
			Amba_HTRANS_Type HTRANS;

	       /**
	         *  @brief Say the type of transaction: if is write or read
			 */   
			bool HWRITE;

	       /**
			 * @brief HBUSREQ Signal: request the Amba Bus for a transaction
			 */   
			bool HBUSREQ;

	       /**
			 * @brief HSIZE Signal: size of data
			 */   
			Amba_HSIZE_Type HSIZE;

	       /**
			 * @brief HBURT Signal: identifies the type of transaction
			 */   
			Amba_HBURST_Type HBURST;

	       /**
			 * @brief HPROT Signal
			 */   
			HPROT_ST HPROT;

	       /**
			 * @brief Default Constructor's of AmbaRequest
			 */   
			AmbaRequest(){
				HADDR = -1;
			}

	       /**
			 * @brief Verifies if the HADDR is empty
	 		 * @return bool Test Status
			 */   
			bool empty(){
				return (HADDR == -1);
			}
			
	       /**
			 * @brief Reset the Amba Resquest
			 */   
			void reset(){
				HADDR = -1;
			} 

	       /**
			 * @brief Operator of Copy
	 		 * @param[in] const AmbaRequest &right: reference for Amba Request that will be copied
	 		 * @return AmbaRequest& Copy of Amba Request
			 */   		
			AmbaRequest& operator=(const AmbaRequest &right){
				this->HADDR = right.HADDR;
				this->HTRANS = right.HTRANS;
				this->HWRITE = right.HWRITE;
				this->HSIZE = right.HSIZE;
				this->HBURST = right.HBURST;
				this->HPROT = right.HPROT;
				return *this;
			}
	};
	
}

#endif /*AMBAREQUEST_H_*/
