/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 */

#ifndef ENUMMASTER_SIG_H_
#define ENUMMASTER_SIG_H_


//Master Transaction Signals
namespace master{

	/**
	 * @brief Represents transactions signals of the AMBA-AHB bus 
	 */
	enum Amba_HTRANS_Type {
		NONSEQUENTIAL = 0,
		SEQUENTIAL,
		IDLE,
		BUSY
	};
	
	/**
	 * @brief Represents the bus data size of the AMBA-AHB bus 
	 */
	enum Amba_HSIZE_Type{
		BYTE = 1,
		HALFWORD = 2,
		WORD = 4,
		SIZE_64 = 8,
		SIZE_128 = 16
	};
	
	/**
	 * @brief Represents the transaction type: simple, burst, etc.
	 */
	enum Amba_HBURST_Type{
		SINGLE = 0,
		INCR,
		WRAP4,
		INCR4,
		WRAP8,
		INCR8,
		WRAP16,
		INCR16
	};
	
	/**
	 * @brief Represents HPROT[0] postition signal : Opcode Fetch || Data Access
	 */
	enum Amba_HPROT_dtOp_Type{
		OPCODE_FETCH = 0,  
		DATA_ACCESS,       
	};
	
	/**
	 * @brief Represents HPROT[1] postition signal : User Access || Privileged Access
	 */
	enum Amba_HPROT_privileged_Type{
		USER_ACCESS,       
		PRIVILEGED_ACCESS, 
	};
	
	/**
	 * @brief Represents HPROT[2] postition signal : Not Bufferable || Bufferable
	 */
	enum Amba_HPROT_bufferable_Type{
		NOT_BUFFERABLE,  
		BUFFERABLE,      
	};
	
	/**
	 * @brief Represents HPROT[2] postition signal : Not Cacheable || Cacheable
	 */ 
	enum Amba_HPROT_cacheable_Type{
		CACHEABLE,         
		NOT_CACHEABLE      
	};
	
	/**
	 * @brief Represents HPROT[3] protection control's signals 
	 */
	struct HPROT_ST{
		Amba_HPROT_dtOp_Type dtOp;
		Amba_HPROT_privileged_Type privileged;
		Amba_HPROT_bufferable_Type bufferable;
		Amba_HPROT_cacheable_Type cacheable;
	};  
	
	
	/**
	 * @brief Represents the transaction type: LOCKED or UNLOCKED 
	 */
	enum Amba_Request_Type{
		LOCKED = 0,
		UNLOCKED
	};

}

#endif /*ENUMMASTER_SIG_H_*/
