#ifndef AMBA_SLAVE_H_
#define AMBA_SLAVE_H_

#include "systemc.h"
#include "tlm.h"
#include "amba_ahb_req.h"
#include "amba_ahb_rsp.h"

using tlm::tlm_transport_if;

struct Amba_AHB_Rsp;

/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief AmbaSlave Class
 */
class AmbaSlave : public virtual tlm_transport_if<Amba_AHB_Req, Amba_AHB_Rsp >,
public sc_module
{

		protected:
	       /**
			 * @brief Id of the Amba Slave
			 */      
			int id;

	       /**
			 * @brief Base of the Amba Slave
			 */      
			unsigned int base;

	       /**
			 * @brief Offset of the Amba Slave
			 */      
			unsigned int offset;

	       /**
			 * @brief Memory Space of the Amba Slave
			 */      
			char* memory;
		
		public:
		
	       /**
			 * @brief AmbaSlave Default Constructor
			 */
			AmbaSlave();
			
	       /**
			 * @brief Default AmbaSlave Constructor
	 		 * @param[in] const AmbaSlave&: referencia for a AmbaSlave Object
			 */      
			AmbaSlave(const AmbaSlave&);

	       /**
			 * @brief AmbaSlave Constructor
	 		 * @param[in] sc_module_name nm: name of the module
			 */      
			AmbaSlave(sc_module_name nm);

	       /**
			 * @brief AmbaSlave Constructor
	 		 * @param[in] sc_module_name nm: name of the module
	 		 * @param[in] int size: size of memory
			 */      
			AmbaSlave(sc_module_name nm, int size);
			
			
			AmbaSlave::AmbaSlave(sc_module_name nm, int base, int offset);
			
	       /**
			 * @brief Returns reference for Default AmbaSlave 
	 		 * @param[in] AmbaSlave* slave_default: reference for Default AmbaSlave 
			 */      
			static void getAmbaSlave_default(AmbaSlave* slave_default);
			
	       /**
			 * @brief Copy Operator
	 		 * @param[in] const AmbaSlave &right: AmbaSlave that will bee copied
	 		 * @return AmbaSlave& destiny of copy
			 */      
			AmbaSlave& operator=(const AmbaSlave &right);

			~AmbaSlave();

	       /**
			 * @brief Returns id of the AmbaSlave 
	 		 * @return char actual id of the AmbaSlave
			 */
			 int getId() const;
			
	       /**
			 * @brief Sets id of the AmbaSlave 
	 		 * @param[in] char id: new id
			 */
			void setId(int id);
			
	       /**
			 * @brief Returns base of the AmbaSlave 
	 		 * @return int base of the AmbaSlave
			 */
			int getBase() const;
			
	       /**
			 * @brief Sets base of the AmbaSlave 
	 		 * @param[in] int base: new base of the AmbaSlave
			 */
			void setBase(int base);
	
    	
    	
    	   /**
			 * @brief Sets offset of the AmbaSlave 
	 		 * @param int offset: offset of the AmbaSlave
			 */
			void setOffset(int offset);

			
	       /**
			 * @brief Returns offset of the AmbaSlave 
	 		 * @return int offset of the AmbaSlave
			 */
			int getOffset() const;
			
	       /**
			 * @brief Returns reference for the memory of the AmbaSlave 
	 		 * @return int reference for memory of the AmbaSlave
			 */
			void getMemory(char*&) const;

	       /**
			 * @brief Transport Method
	 		 * @param[in] const Amba_AHB_Req& amba_ahb_req: packaqe that will be sent to transport layer
	 		 * @return Amba_AHB_Rsp response package coming from the transport layer
			 */      
			virtual	Amba_AHB_Rsp transport(const Amba_AHB_Req& amba_ahb_req);
			
} ;

#endif /*AMBA_SLAVE_H_*/
