#ifndef AMBA_TL1_SLAVE_IF_
#define AMBA_TL1_SLAVE_IF_

#include "amba_data.h"
#include "amba_slave.h"
#include "amba_request.h"
#include "amba_response.h"

using namespace master;
using namespace amba;
/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief slave Namespace
 */
namespace slave{

/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief Amba_slave_tl1_if Class
 */
	class Amba_Slave_tl1_if : virtual public sc_interface {
		public:
		

			/**
			 * @brief Slave receives the control signals from Amba Bus
	 		 * @param[in] AmbaMaster& master: reference for a AmbaMaster Object
	 		 * @param[in] Amba_Request_Type req: request type
	 		 * @return bool Request Status
			 */   			
			virtual void slave_receive_controls_signals(AmbaSlave& slave, AmbaRequest& ambaRequest) = 0;			
	
	       /**
			 * @brief Reads data signals from Amba Bus
	 		 * @param[in] AmbaSlave& slave: reference for a AmbaSlave Object
	 		 * @param[in] AmbaResponse& ambaResponse: response of the slave_read_data operation
	 		 * @param[in] AmbaDara& data: referece for data that will be received
			 */   
			virtual void slave_read_data(AmbaSlave& slave, AmbaResponse& response, AmbaData& data) = 0;
		
	       /**
			 * @brief Sends data signals to Amba Bus
	 		 * @param[in] AmbaMaster& slave: reference for a AmbaSlave Object
	 		 * @param[in] AmbaResponse& ambaResponse: response of the slave_send_data operation
	 		 * @param[in] AmbaDara& data: referece for data that will be transmitted
			 */   
			virtual void slave_send_data(AmbaSlave& slave, AmbaResponse& response, AmbaData& data) = 0;
			
	       /**
			 * @brief Signals that Split Operation ends
	 		 * @param[in] AmbaSlave* slave: reference for a AmbaSlave Object
			 */   
			virtual void slave_end_split(AmbaSlave& slave) = 0;
			

	       /**
			 * @brief Registers the Amba Slave in Amba Bus
	 		 * @param[in] AmbaSlave* slave: pointer for a AmbaSlave Object
			 */   
			virtual void register_slave(AmbaSlave* slave) = 0;
	};
}

#endif /*AMBA_TL1_SLAVE_IF_*/
