#ifndef WRAPPERMASTERAMBATLM_
#define WRAPPERMASTERAMBATLM_
#include "tlm.h"
#include "amba_master_tl1_if.h"
#include "amba_ahb_req.h"
#include "amba_ahb_rsp.h"
#include "systemc.h"
#include "amba_bus.h"
#include "amba_master.h"

using tlm::tlm_transport_if;

/**
 * @brief Wrapper that translate Amba protocol to TLM protocol
 * @author Andre Silva
 * @author Guilherme Alvaro
 */


class WrapperMasterAmbaTLM: public sc_module, 
public virtual tlm_transport_if<Amba_AHB_Req, Amba_AHB_Rsp>{
	
	private:
		/**
		 * @brief Port to bus
		 */
		sc_port<Amba_Master_tl1_if > portBus;
		SC_HAS_PROCESS(WrapperMasterAmbaTLM);

		/**
		 * @brief Pointer to AmbaMaster
		 */
		AmbaMaster* master;
	public:

		/**
		 * @brief sc_export to the WrapperMasterAmbaTLM
		 */
		sc_export<tlm_transport_if<Amba_AHB_Req, Amba_AHB_Rsp> > portMaster;

		/**
		 * @brief The transport method of TLM
		 * @param Amba_AHB_Req& amba_ahb_req: the TLM request package
		 * @return Amba_AHB_Rsp : the TLM response package
		 */
		Amba_AHB_Rsp transport(const Amba_AHB_Req& amba_ahb_req);	

		/**
		 * @brief Constructor of WrapperMasterAmbaTLM
		 * @param sc_module_name nm: name of wrapper
		 * @param AmbaBus* bus: pointer to the AmbaBus
		 * @param AmbaMaster*: pointer to teh AmbaMaster
		 */
		WrapperMasterAmbaTLM(sc_module_name nm, AmbaBus* bus, AmbaMaster* master); 
		
		bool end_transport(const Amba_AHB_Rsp& rsp){
			return (((rsp.amba_response.HRESP == OKAY) || (rsp.amba_response.HRESP == ERR))
			         && rsp.amba_response.HREADY == true);
		}
};


#endif /*WRAPPERMASTERAMBATLM_*/
