#ifndef WRAPPERSLAVEAMBATLM_
#define WRAPPERSLAVEAMBATLM_
#include "tlm.h"
#include "amba_ahb_req.h"
#include "amba_ahb_rsp.h"
#include "systemc.h"
#include "amba_slave.h"
#include "amba_slave_tl1_if.h"
#include "amba_bus.h"


using namespace slave;
using namespace amba; 
using tlm::tlm_transport_if;

/**
 * @brief Wrapper that simulates a Slave Amba and  Amba protocol to TLM protocol
 * @author Andre Silva
 * @author Guilherme Alvaro
 */


class WrapperSlaveAmbaTLM: public sc_module {
	
	private:
	/**
	 * @brief The port to bus
	 */
		sc_port<Amba_Slave_tl1_if > portBus;
	/**
	 * @brief The port to slave
	 */
		sc_port<tlm_transport_if<Amba_AHB_Req, Amba_AHB_Rsp> > portSlave;

		SC_HAS_PROCESS(WrapperSlaveAmbaTLM);

	/**
	 * @brief Pointer to the slave
	 */
		AmbaSlave* slave;

	/**
	 * @brief Indicates if is happening a split transaction
	 */
		bool is_split;

	/**
	 * @brief Indicates if exists a transport operation in course
	 */
		bool is_transport;

	/**
	 * @brief Indicates the initiate of the transport operation
	 */
		sc_event transport_event;

	/**
	 * @brief Indicates the end of the transport operation
	 */
		sc_event resp_transport_event;

	/**
	 * @brief The Request TLM Package
	 */
		Amba_AHB_Req req;

	/**
	 * @brief The Response TLM Package
	 */
		Amba_AHB_Rsp rsp;

	/**
	 * @brief The Response Amba TL1 Package
	 */
		AmbaResponse resp;
		

		
	/**
	 * @brief Thread that simulates the slave amba
	 */
		void simulate_slave_thread();

	/**
	 * @brief Configures an default Response Amba TL1 Package
	 */
		void createDefaultResponse();

	/**
	 * @brief Thread that makes the transport transaction
	 */
		void simulate_transport_thread();

	/**
	 * @brief Configures an retry Response Amba TL1 Package
	 */
	 void createRetryResponse();
	
	public:
	

	/**
	 * @brief Pointer to the arbiter
	 */
		WrapperSlaveAmbaTLM(sc_module_name nm, AmbaBus* bus, AmbaSlave* slave);
		
};

#endif /*WRAPPERSLAVEAMBATLM_*/
