#ifndef AMBADATA_H_
#define AMBADATA_H_ 

#include "util.h"
#include "systemc.h"
#include "iostream.h"
#include "ac_tlm_protocol.H"

using namespace util; 
/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief Amba Namespace
 */
namespace amba{

/**
 * @author Andre Silva
 * @author Guilherme Alvaro
 * 
 * @brief Represents the data that will be trasmitted into Amba Bus
 */
class AmbaData {


#define DEFAULT_SIZE_BYTE 4
#define DEFAULT_SIZE_MAX_BYTE 128
		
	private:

		/* * 
		 * @brief Size of Data that will be transmitted into Amba Bus
		 */		
		int size;
	
	public:	

		/* * 
		 * @brief Data that will be transmitted into Amba Bus
		 */	
		char* dado;
//		int dado;

		/* * 
		 * @brief AmbaData Contructor's
		 * @param[in] int size: size of data
		 */	
		AmbaData (int size){
			if (size > DEFAULT_SIZE_MAX_BYTE)
			{

				dado = new char[DEFAULT_SIZE_MAX_BYTE];
				this->size = DEFAULT_SIZE_MAX_BYTE;
			}
			else
				if(size < 0){
					dado = new char[DEFAULT_SIZE_BYTE];
					this->size = DEFAULT_SIZE_BYTE;
				}
				else{
					dado = new char[util::exponentialBase2(size)];
					this->size = util::exponentialBase2(size);
				}
//			this->size = size; 

		}

		/* * 
		 * @brief AmbaData Default Contructor's
		 * @param[in] int size: size of data
		 */			
		AmbaData (){
			dado = new char[DEFAULT_SIZE_BYTE];
//			dado = 0;
			this->size = DEFAULT_SIZE_BYTE;
		}
		
		/* * 
		 * @brief Gets the size of the data
		 * @return int size of data
		 */			
		 int getSize() const{
			return this->size;
		}
		
		/* * 
		 * @brief Sets the size of the data
		 * @param[in] int size: new size of data
		 */					
		 void setSize(int size) {
			char* temp = &(*dado);
			delete[] dado;
			dado = new char[size];	
			for (int i = 0; i < size && i < this->size; i++)
				dado[i] = temp[i];
//			this->size = size;
		}
		
		/* * 
		 * @brief Copy Operator
		 * @param[in] const AmbaData & right: data that will be copied
		 * @return: AmbaData copy of data
		 */							
		 const AmbaData &operator=(const AmbaData & right){
			if(dado != NULL)	 
			{
				delete[] dado;
				dado = NULL;
			}
			this->size = right.getSize();
			dado = new char[this->size]; 
			for (int i=0; i < this->size; i++){
				dado[i] = right.dado[i];
			}
//			dado = right.dado;
			return *this; 
		}
		
		//char* = uint32_t
		void copiaDado(const uint32_t &d){
	
			if(dado != NULL){
				delete[] dado;
				dado = NULL;
			}
			dado = new char[4];
			dado[0] = (d & 0x000000ff);
			dado[1] = (d & 0x0000ff00) >> 8;
			dado[2] = (d & 0x00ff0000) >> 16;
			dado[3] = (d & 0xff000000) >> 24;
		}	

		// uint32_t = char*
		void getDado(uint32_t&  d){
			uint32_t temp0 = (unsigned int)dado[0];
			uint32_t temp1 = (unsigned int)dado[1] << 8;
			uint32_t temp2 = (unsigned int)dado[2] << 16;
			uint32_t temp3 = (unsigned int)dado[3] << 24;
			temp0 &= 0x000000ff;
			temp1 &= 0x0000ff00; 
			temp2 &= 0x00ff0000; 
			temp3 &= 0xff000000; 
			d ^= d;
			d |= temp0;
			d |= temp1;
			d |= temp2;
			d |= temp3;
		}

	
	};
	
};

#endif /*AMBADATA_H_*/
